/*
 * Project:	chkonline (add on tool for isdn4linux)
 * Author:	Friedemann Baitinger (baiti@herrenberg,netsurf.de)
 * Date:	95/12/30
 *
 * Copyright 1996 by Friedemann Baitinger (baiti@herrenberg,netsurf.de) 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Description:
 *
 *	The purpose of this program is to provide a mechanism to check
 *	whether a network type connection to a specific destination
 *	exists at the time the progam is called. 'ping' or anything
 *	like that can't be used because using it would immediately
 *	trigger a connect request. Consequently we have to read the
 *	status data from the /dev/isdnmon device and parse the data
 *	accordingly. 
 *
 *	Fortunately Michael Knigge already wrote a program (IMON) which
 *	does the reading and parsing of the data supplied by /dev/isdnmon.
 *	I took the liberty to reuse his code.
 *
 *	Usage:  chkonline phone_number
 *
 *	the tool returns 0 if a network type connection to the given
 *	destination phone number exists. otherwise a non-zero return
 *	code is returned. 'phone_number' doesn't have to be a fully
 *	qualified number, a valid substring would do just fine, however
 *	wildcard handling is not supported at this time and probably
 *	will never be....
 *
 *  $Log: chkonline.c,v $
 *  Revision 1.6  1997/05/09 15:08:14  baiti
 *  Wolfram added the '-d in|out' logic 'bare bones', I beefed it up a bit in
 *  order to allow easy addition of further commandline options.
 *
 *  Revision 1.5  1997/04/20 12:00:23  baiti
 *  Return code checking logic was changed by Wolfram but the 'block comment'
 *  above it was still there. I've just removed it.
 *
 *  Revision 1.4  1997/04/20 11:57:34  baiti
 *  'rcsid' added
 *
 *  Revision 1.3  1997/04/20 11:52:31  baiti
 *  'Release' dropped from module header, this is no being taken care of by
 *  CVS revision numbers.
 *
 *  Revision 1.2  1997/04/20 11:50:50  baiti
 *  Bug fix: Only the first ISDN channel which was found online, was searched
 *  for the given telephone number. Obviously I never had more than one
 *  connection active while testing the 'toy'. This bug was reported and
 *  fixed by:  Wolfram.Pienkoss@holzland.thur.de
 *
 *  This fix was also reason enough to check the stuff into a CVS repository
 *
 */

#define TESTx
#define BUF_SIZ			512

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>

#ifndef ISDN_MAX_CHANNELS
    #define ISDN_MAX_CHANNELS	4
    #define ISDN_USAGE_NET	3
    #define ISDN_USAGE_OUTGOING 128
#endif
   
static char *rcsid = "@(#) $Id: chkonline.c,v 1.6 1997/05/09 15:08:14 baiti Exp $";

static char idmap_line[BUF_SIZ];
static char chmap_line[BUF_SIZ];
static char drmap_line[BUF_SIZ];
static char usage_line[BUF_SIZ];
static char flags_line[BUF_SIZ];
static char phone_line[BUF_SIZ];

static FILE *isdninfo;


void usage(char *appl)
{
    printf("Usage: %s [-d in|out] phone_number\n", appl);
    exit(-1);
}


int main(int argc, char **argv) 
{
    char *ptr_idmap;
    char *ptr_chmap;
    char *ptr_drmap;
    char *ptr_usage;
    char *ptr_flags;
    char *ptr_phone;
    char *my_phone;
   
    char channel[32];
    char phone[32];

    char temp[32];   

    char *direction = NULL;

    const char *zopts   = "d:";
    const char *dir_in  = "in";
    const char *dir_out = "out";

    int  local;
    int  chanum;
    int  line;
    int  iopt;

    int  rc = 1;

    while ((iopt = getopt(argc, argv, zopts)) != EOF) {
        switch (iopt) {
            case 'd':
                direction = optarg;
                if (direction == NULL)
                    usage(argv[0]);
                if (strcmp(dir_in, direction) != 0 ) {
                    if ( strcmp(dir_out, direction) != 0 ) {
                        fprintf(stdout, "%s: invalid direction\n", argv[0]);
                        usage(argv[0]);
                    }
                }
                break;
            default:
                fprintf(stdout, "%s: undecoded option detected\n", argv[0]);
		exit(-1);
                break;
	}
    }

    my_phone = argv[optind];

    if (my_phone == NULL) {
	fprintf(stdout, "missing phone number\n");
	usage(argv[0]);
    }


    if (!(isdninfo = fopen("/dev/isdninfo", "r"))) {
        fprintf(stderr, "Can't open /dev/isdninfo\n");
        exit(-2);
    }
   
    fgets(idmap_line, BUF_SIZ, isdninfo);
    fgets(chmap_line, BUF_SIZ, isdninfo);
    fgets(drmap_line, BUF_SIZ, isdninfo);
    fgets(usage_line, BUF_SIZ, isdninfo);
    fgets(flags_line, BUF_SIZ, isdninfo);
    fgets(phone_line, BUF_SIZ, isdninfo);
    
#ifdef TEST1
    sprintf(idmap_line, "idmap: TA250034 - - - - - - - - - - - - - - -");
    sprintf(chmap_line, "chmap: 0 1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1");
    sprintf(drmap_line, "drmap: 0 0 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1");   
    sprintf(usage_line, "usage: 1 2 0 3 4 0 5 0 129 0 130 131 132 0 0 133");
    sprintf(flags_line, "flags: 0 ? ? ? ? ? ? ? ? ? ? ? ? ? ? ?");
    sprintf(phone_line, "phone: 05114711 0815 ??? 5555 666666666 ??? 4711 ??? 875257 ??? 77848 7890 2345467 ??? ??? 8888888888");

#endif
#ifdef TEST2
    sprintf(idmap_line, "idmap:  teles0 teles0 - - - - - - - - - - - - - -"); 
    sprintf(chmap_line, "chmap:  0 1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1"); 
    sprintf(drmap_line, "drmap:  0 0 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1 -1"); 
    sprintf(usage_line, "usage:  131 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0"); 
    sprintf(flags_line, "flags:  1 ? ? ? ? ? ? ? ? ? ? ? ? ? ? ?"); 
    sprintf(phone_line, "phone:  87194 ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ??? ???"); 

#endif

    ptr_idmap = idmap_line + 7;
    ptr_chmap = chmap_line + 7;
    ptr_drmap = drmap_line + 7;
    ptr_usage = usage_line + 7;
    ptr_flags = flags_line + 7;
    ptr_phone = phone_line + 7;
   
    for (line = 1; line < ISDN_MAX_CHANNELS + 1; line++) {
        sscanf(ptr_idmap, "%s", channel);
        ptr_idmap += strlen(channel) + 1;
        sscanf(ptr_usage, "%d", &local);   
        sscanf(ptr_usage, "%s", temp);
        ptr_usage += strlen(temp) + 1;
        sscanf(ptr_phone, "%s", phone);
        sscanf(ptr_chmap, "%d", &chanum);
        sscanf(ptr_chmap, "%s", temp);
        ptr_chmap = ptr_chmap + strlen(temp) + 1;

        if (chanum == -1)
            continue;

        if ((local & 7) == 0) {
            ptr_phone += 4;
            continue;
        }
        ptr_phone += strlen(phone) + 1;

        if ((local & 7) != ISDN_USAGE_NET)
            continue;

	/*
	 * here we believe we have found a channel with a network type
	 * connection.
	 */
	 
        if (strstr(phone, my_phone) != NULL) {
	    rc = 0;
	    if (direction == NULL) 
                break;
	    if ((((local & ISDN_USAGE_OUTGOING) == 0)  && \
		  (strcmp(dir_out, direction)   == 0)) || \
		(((local & ISDN_USAGE_OUTGOING) != 0)  && \
		  (strcmp(dir_in, direction)    == 0))) {
		rc = 1;
	    }
	    break;
	}
    }
     
#if defined(TEST1) | defined(TEST2)            
    printf("rc = %d\n", rc);
#endif            
    return(rc);
}
